#pragma once

#include <lektor/defines.h>
#include <lektor/mkv.h>
#include <lektor/thread.h>
#include <curl/curl.h>
#include <sqlite3.h>
#include <json.h>
#include <inttypes.h>
#include <stdbool.h>

struct lkt_repo {
    const char *name;
    const char *base_url;
    const char *kara_dir;
    const char *get_all_json;
    const char *get_id_json;
    const char *get_id_file;
    const uint64_t version;
};

int repo_new(struct lkt_repo *const repo, const char *name, const char *url);
void repo_free(struct lkt_repo *const repo);

/* Only one possible repo thread is authorized. */
int repo_new_thread(struct lkt_repo *const repo);
int repo_join_thread(void);

/* Get metadata of a kara. */
int repo_get_id(struct lkt_repo *const repo, const uint64_t id, struct kara_metadata *mdt);
int repo_get_alljson_sync(struct lkt_repo *const repo, struct json_object **json);
int repo_get_allid_async(void);

/* Download a kara. */
int repo_download_id_sync(struct lkt_repo *const repo, sqlite3 *db, const uint64_t id,
                          const char *kara_path, struct kara_metadata *mdt_ret);
int repo_download_id_async(const size_t id);
int repo_get_kara_async(struct kara **downloaded);
