#pragma once

#include <lektor/defines.h>
#include <pthread.h>
#include <sys/types.h>

struct lkt_thread {
    /* The input poll. */
    volatile unsigned int input_len;
    volatile unsigned int input_size;
    volatile void *volatile *volatile input_cells;
    pthread_mutex_t input_lock;

    pthread_t th;
    unsigned int initial_size;

    /* The output pool. */
    volatile unsigned int output_len;
    volatile unsigned int output_size;
    volatile void *volatile *volatile output_cells;
    pthread_mutex_t output_lock;
};

struct lkt_thread_arg {
    struct lkt_thread *self;
    void *args;
};


/* Create a new thread. Returns 0 on success, a non zero value on error. */
int lkt_th_new(struct lkt_thread *th, unsigned int init_sizes,
               void *(*func)(struct lkt_thread_arg *), void *args);

/* Joins a thread. Returns 0 on success, a non zero value on error. */
int lkt_th_join(struct lkt_thread *th, void **ret);

/* Input qeueu manipulations. */
int lkt_th_append_input(struct lkt_thread *th, void *ptr);
int lkt_th_pop_input(struct lkt_thread *th, void **ptr);
int lkt_th_head_input(struct lkt_thread *th, void **ptr);

/* Output qeueu manipulations. */
int lkt_th_append_output(struct lkt_thread *th, void *ptr);
int lkt_th_pop_output(struct lkt_thread *th, void **ptr);
int lkt_th_head_output(struct lkt_thread *th, void **ptr);
