#define _POSIX_C_SOURCE 200809L

#include <lektor/net.h>

#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <limits.h>

struct lkt_command
lkt_command_parse(char *raw)
{
    errno = 0;
    char *endptr;
    struct lkt_command res = { .name = raw, .args = {0}, .cont = strtol(raw, &endptr, 10) };

    if ((errno == ERANGE && (res.cont == LONG_MAX || res.cont == LONG_MIN)) || (errno != 0)) {
        res.cont = 0;
        goto skip_cont;
    }

    if (endptr == raw) {
        res.cont = 0;
        goto skip_cont;
    }

    if (*endptr == '\0') {
        /* That thing is an error. */
        return res;
    }

    /* Get the real command name. */
    raw += strspn(raw, " 1234567890");
    if (!*raw)
        return res;
    res.name = raw;

    /* Get the args. */
skip_cont:
    raw += strcspn(raw, " ");
    if (!*raw)
        return res;

    *raw++ = 0;
    raw += strspn(raw, " ");

    for (size_t i = 0; i < LKT_MESSAGE_ARGS_MAX && *raw; i++) {
        if (*raw == '"') {
            res.args[i] = ++raw;
            raw += strcspn(raw, "\"");
            if (*raw) {
                *raw++ = 0;
                raw += strcspn(raw, " ");
                *raw++ = 0;
            }
        } else if (*raw == '\'') {
            res.args[i] = ++raw;
            raw += strcspn(raw, "'");
            if (*raw) {
                *raw++ = 0;
                raw += strcspn(raw, " ");
                *raw++ = 0;
            }
        } else {
            res.args[i] = raw;
            raw += strcspn(raw, " ");
            *raw++ = 0;
        }
        raw += strspn(raw, " ");
    }

    return res;
}

struct lkt_command
lkt_command_clone(struct lkt_command *c)
{
    struct lkt_command res = {
        .name = strdup(c->name),
        .args = {0},
    };

    for (size_t i = 0; c->args[i] && i < LKT_MESSAGE_ARGS_MAX; i++)
        res.args[i] = strdup(c->args[i]);

    return res;
}

void
lkt_command_free(struct lkt_command *c)
{
    free(c->name);
    for (size_t i = 0; c->args[i]; i++)
        free(c->args[i]);
}
